; director.asm
;  allows you to show a scene
;   - scroll map
;   - move characters, NPCs
;   - call special routines

directorPointer .equ appBackUpScreen+62
directorCharTable .equ appBackUpScreen+64
;FORMAT
;0 db <character sprite id (1st sprite is 2)> 1=your char
;1 db <direction>
;2 db <initial X (0-80)>
;3 db <initial Y (0-48)>

cinemaTable .equ appBackUpScreen+96
;ENTRY
; db <type>, <3 bytes... based on type>

;TYPES
;  0 = stop
;1		extra bytes empty

;  1 = redraw screen
;1		delay time in # of HALTs

;  2 = scroll screen  (automatically scroll the characters)
;1   scroll cycles (0 = only once)
;2   x scroll
;3   y scroll

;  3 = move char
;1		character ID (in directorCharTable)
;2   character direction (0=north, 1=e, 2=s, 3=w)
;3   steps in that direction

;  4 = add char
;1 char id
;2 xpos
;3 ypos

forestChase:
 ld a,12
 ld (mapWidth),a
 ld a,15
 ld (mapHeight),a
 ld a,2
 ld (mapX),a
 ld a,10
 ld (mapY),a
 ld a,255
 ld (directorCharTable),a
 ld a,3
 ld (mapID),a
 
 ld hl,forestChase1_cinema
 ld (directorPointer),hl

 ld hl,forestMap
 jr director

rebelAttack:
 ld a,14
 ld (mapWidth),a
 ld a,17
 ld (mapHeight),a
 xor a
 ld (mapX),a
 ld (mapY),a
 ld a,255
 ld (directorCharTable),a
 ld a,1
 ld (mapID),a
 
 ld hl,rebelAttack1_cinema
 ld (directorPointer),hl

 ld hl,castleMap
 
director:
 push hl
  call loadAppvarInDE
 pop hl
 add hl,de
 ld (mapStart),hl

 ld de,cinemaTable
 ld hl,(directorPointer)
 ld bc,256
 ldir
 
 ld hl,cinemaTable
 ld (directorPointer),hl

directorLoop:
  ld a,(charTimer)
  inc a
  ld (charTimer),a
  B_CALL GrBufClr

  call drawmap

; loop through director char table
 ld hl,directorCharTable
directorLoopChars:
 ld a,(hl)
 cp 255
 jr z,exitDirectorChars
 ld (charMovingSprite),a

 inc hl
 ld a,(hl)
 ld (charDir),a
 inc hl
 ld a,(hl)
 ld (charX),a
 inc hl
 ld a,(hl)
 ld (charY),a

 push hl
  call showCharacter
 pop hl
 inc hl
 jr directorLoopChars

exitDirectorChars:
 B_CALL grbufcpy  

 ei
 halt
 halt
 halt
 halt
 halt

 ld hl,(directorPointer)
 ld a,(hl)
 or a
 ret z

 cp 2
 jr z,directorScroll

 cp 3
 jr z,directorMoveChar

 cp 4
 jp z,addCharacterToCinema

 cp 5
 jp z,directorFunctionCall

 cp 6
 jp z,directorDialog

directorNext:
 ld de,4
 ld hl,(directorPointer)
 add hl,de
 ld (directorPointer),hl
 jr directorLoop

directorScroll:
 inc hl
 ld a,(hl)
 or a
 jr z,directorNext
 dec a
 ld (hl),a
 inc hl
 ld b,(hl)
 dec b
 dec b
 ld a,(mapX)
 add a,b
 ld (mapX),a
 inc hl
 ld b,(hl)
 dec b
 dec b
 ld a,(mapY)
 add a,b
 ld (mapY),a
 call dirDelay
 jp directorLoop


directorMoveChar:
 inc hl
 ld b,(hl)
 call findDirectorChar
 jr nc,directorNext			; cant find that char

 ex de,hl
 ld hl,(directorPointer)
 inc hl
 inc hl

 inc de
 ld a,(hl)
 ld (de),a
 ld b,a

 inc hl
 ld a,(hl)
 sub 2
 jr c,directorNext

 ld (hl),a
 
  ; determine the new x/y pos based on the direction
 ex de,hl
 inc hl

 ld a,b
 cp 2
 jr z,directorRight
 cp 1
 jr z,directorUp
 or a
 jr z,directorLeft

 inc hl

 inc (hl)
 inc (hl)
 jp directorLoop

directorUp:
 inc hl

 dec (hl)
 dec (hl)
 jp directorLoop

directorLeft:
 dec (hl)
 dec (hl)
 jp directorLoop

directorRight:
 inc (hl)
 inc (hl)
 jp directorLoop


findDirectorChar:
; loop through director char table
 ld hl,directorCharTable
 ld de,4
directorCharsEnd:
 ld a,(hl)
 cp b
 jr z,directorCharFOUND
 add hl,de
 jr directorCharsEnd

directorCharFOUND:
 scf
 ret

addCharacterToCinema:
 inc hl
 ; set DE = end of directorCharTable
 push hl
  ld b,255			
  call findDirectorChar
  ex de,hl
 pop hl
 ld a,(hl)
 ld (de),a
 inc de
 ld a,1
 ld (de),a
 inc de
 inc hl
 ld a,(hl)
 ld (de),a
 inc hl
 inc de
 ld a,(hl)
 ld (de),a

 inc de
 ld a,255
 ld (de),a
 jp directorNext

directorDialog:
 inc hl
 ld e,(hl)
 inc hl
 ld d,(hl)
 ex de,hl
 call showDialog
 jp directorNext

directorFunctionCall:
 inc hl
 ld e,(hl)
 inc hl
 ld d,(hl)
 inc hl
 ld b,(hl)
 ex de,hl
 jp (hl)

rebelWallBustFUNCTION:
 ; find the wall tiles and bust them

 call loadAppvarInDE
 ld hl,castleMap+45
 add hl,de

 ld (hl),035h
 inc hl
 ld (hl),03Fh
 inc hl
 ld (hl),036h
 jp directorNext

directorRemoveChars:
 call findDirectorChar
 ld (hl),255
 jp directorNext

dirDelay:
 ei
 ld b,40
dirDelay2:
 halt
 djnz dirDelay2
 ret

; with 2 (scroll map), 1=1 left, 2=nothing, 3=1 right
rebelAttack1_cinema:
 db 2,6,3,4
 db 4,0,72,38
 db 3,0,0,6
 db 2,3,1,2

 db 3,0,0,6
 db 3,0,1,0
 db 2,3,2,1

 db 3,0,0,0
 db 2,3,1,2
 db 3,0,0,60
 db 3,0,1,0
 db 2,4,2,1
 db 3,0,2,12
 db 3,0,1,0
 db 2,3,2,1

 ; WALL BUSTS

 db 6
 dw rebelAttackDialog1
 db 0

 db 5
 dw rebelWallBustFUNCTION
 db 0

 db 4,3,48,14
 db 3,3,3,12

 db 4,2,36,26
 db 4,4,48,38

 db 6
 dw rebelAttackDialog2
 db 0

 db 3,4,3,12
 db 3,4,2,12
 db 3,4,1,0

 db 3,2,3,24
 db 3,2,2,24
 db 3,2,1,0

 db 5
 dw directorRemoveChars
 db 2

 ; pause and reflect

 db 1,50,0,0

 db 6
 dw rebelAttackDialog3
 db 0

 ; come outside w/ treasure chest
 
 db 4,2,60,50
 db 3,2,0,24
 db 3,2,1,12

 db 5
 dw directorRemoveChars
 db 2

 db 4,2,60,50
 db 1,40,0,0

 db 4,6,48,50
 db 1,40,0,0

 db 3,2,0,24

 db 5
 dw directorRemoveChars
 db 2

 db 3,3,1,12
 db 1,40,0,0

 db 5
 dw directorRemoveChars
 db 3

 ; car leaves and guards run out

 db 6
 dw rebelAttackDialog4
 db 0

 db 4,1,60,50
 db 3,1,0,12
 db 3,1,1,12

 db 4,5,60,50
 db 3,5,0,12
 db 3,5,1,24

 db 5
 dw directorRemoveChars
 db 1

 db 2,1,2,1

 db 3,0,0,12
 db 3,0,1,36

 db 2,1,2,1


 db 0

rebelAttackDialog1:
 db "GRRRRRR!!!",3
 db "KA-BOOM!",0

rebelAttackDialog2: 
 db "Rebel: Lets get our",2
 db "treasure back",0

rebelAttackDialog3:
 db "You: (I can stop them)",0

rebelAttackDialog4:
 db "HALT THIEF!",0

forestChase1_cinema:
 db 4,1,36,50
 db 3,1,1,24
 db 3,1,2,12
 db 4,5,36,50
 db 3,5,1,24
 db 4,0,36,50
 db 3,1,1,0

 db 6
 dw forestDialog1
 db 0

 db 2,2,2,1

 db 6
 dw forestDialog2
 db 0

 db 3,1,3,0
 db 3,5,3,0

 db 6
 dw forestDialog3
 db 0

 db 5
 dw directorRemoveChars
 db 2

 db 4,0,36,38

 db 0

forestDialog1:
 db "Dirk: Wait up",2
 db "I want to help",0

forestDialog2:
 db "Hawk: If you must",2
 db "Snake: but hurry up",0

forestDialog3:
 db "Hawk and Snake",2
 db "joined.",0
